package com.example.android.livewallpaper2;

import java.nio.IntBuffer;

import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.util.Log;

public class FireEffect {
    static {
        System.loadLibrary("FireEffect");
    }
    static int boost = 0;
    boolean useNative = true;
    
    private int color[];
    private int width;
    private int height;
    private int image[];
    private IntBuffer buffer;
    private int pallet[];
    private int seedparam[];
    private Bitmap bitmap;
    private int fireLevel = 120;
    
    public void init(int w, int h, int fireLevel) {
        synchronized (this) {
            width = w == 0 ? width : w;
            height = h == 0 ? height : h;
            this.fireLevel = fireLevel;
            pallet = new int[width * height];
            
            color = new int[128];
            for (int i = 0, index = 0; i < 32; i++, index++) color[index] = Color.rgb(i * 8, 0, 0);
            for (int i = 0, index = 32; i < 32; i++, index++) color[index] = Color.rgb(255, i * 8, 0);
            for (int i = 0, index = 64; i < 64; i++, index++) color[index] = Color.rgb(255, 255, i * 4);
            
            seedparam = new int[width * height];
            
            for (int i = (height - 2) * width; i < width * height; i++) {
                seedparam[i] = (int)(128 * Math.random());
            }
            
            image = new int[width * height];
            
            buffer = IntBuffer.allocate(width * height);
            buffer.put(image);
            bitmap = Bitmap.createBitmap(width, height, Bitmap.Config.RGB_565);
            
            long l = System.currentTimeMillis();
            String s = hello();
            Log.d("Hello", s + ":" + (System.currentTimeMillis() - l));
        }
    }
    
    public void draw(Canvas canvas) {
        synchronized (this) {
            long l =  System.currentTimeMillis();
            if (useNative) {
                long o = System.currentTimeMillis();
                effect(fireLevel, width, height, image, pallet, seedparam, color);
                Log.d("Overhead", (System.currentTimeMillis() - o) + "ms");
            } else {
                for (int i = 0; i < width * height; i++) {
                    if (seedparam[i] != 0 && seedparam[i] > (int)((fireLevel - boost) * Math.random())) {
                        pallet[i] = 127;
                    } else if (seedparam[i] != 0) {
                        pallet[i] = 0;
                    }
                }
                if (boost > 0) {
                    boost--;
                }
                for (int i = 1; i < height - 1; i++) {
                    for (int j = 1; j < width - 1; j++) {
                        pallet[(i - 1) * width + j] = (pallet[i * width + j] + 
                                pallet[i * width + j - 1] + 
                                pallet[i * width + j + 1] + 
                                pallet[(i - 1) * width + j] + 
                                pallet[(i + 1) * width + j]) / 5;
                    }
                }
                for (int i = 0; i < width * height; i++) {
                    image[i] = color[pallet[i]];
                }
            }
            bitmap.setPixels(image, 0, width, 0, 0, width, height);
            canvas.drawBitmap(bitmap, 0, 0, null);
            Log.d("TIME", (useNative ? "JNI:  " : "Java: ") + (System.currentTimeMillis() - l) + "ms");
        }
    }
    
    private native void effect(int fireLevel, int width, int height, int[] image, int[] pallet, int[] seedparam, int[] color);
    private native String hello();
}
