package com.example.android.livewallpaper2;

import android.content.SharedPreferences;
import android.graphics.Canvas;
import android.os.Handler;
import android.service.wallpaper.WallpaperService;
import android.util.Log;
import android.view.MotionEvent;
import android.view.SurfaceHolder;

public class FireWallpaperService extends WallpaperService {
    
    public static final String SHARED_PREFS_NAME = "com.example.android.livewallpaper2";
    
    private final Handler handler = new Handler();
    
    @Override
    public Engine onCreateEngine() {
        return new FireEffectEngine();
    }
    
    class FireEffectEngine extends Engine implements SharedPreferences.OnSharedPreferenceChangeListener {
        private boolean visible;
        private FireEffect fireEffect;
        private SharedPreferences prefs;
        
        private final Runnable drawFire = new Runnable() {
            @Override
            public void run() {
                drawFrame();
            }
        };
        
        public FireEffectEngine() {
            fireEffect = new FireEffect();
            prefs = FireWallpaperService.this.getSharedPreferences(SHARED_PREFS_NAME, 0);
            prefs.registerOnSharedPreferenceChangeListener(this);
            onSharedPreferenceChanged(prefs, null);
        }
        
        @Override
        public void onCreate(SurfaceHolder surfaceHolder) {
            super.onCreate(surfaceHolder);
            Log.d("TEST", "onCreate");
            setTouchEventsEnabled(true);
        }
        
        @Override
        public void onDestroy() {
            super.onDestroy();
            Log.d("TEST", "onDestroy");
            handler.removeCallbacks(drawFire);
        }
        
        @Override
        public void onSurfaceCreated(SurfaceHolder holder) {
            super.onSurfaceCreated(holder);
            Log.d("TEST", "onSurfaceCreated");
        }
        
        @Override
        public void onSurfaceChanged(SurfaceHolder holder, int format, int width, int height) {
            super.onSurfaceChanged(holder, format, width, height);
            Log.d("TEST", "onSurfaceChanged");
            onSharedPreferenceChanged(prefs, null);
            drawFrame();
        }
        
        @Override
        public void onSurfaceDestroyed(SurfaceHolder holder) {
            super.onSurfaceDestroyed(holder);
            Log.d("TEST", "onSurfaceDestroyed");
            visible = false;
            handler.removeCallbacks(drawFire);
        }
        
        @Override
        public void onDesiredSizeChanged(int desiredWidth, int desiredHeight) {
            Log.d("TEST", "onDesiredSizeChanged:desiredWidth=" + desiredWidth + ", desiredHeight=" + desiredHeight);
            onSharedPreferenceChanged(prefs, null);
            drawFrame();
        }
        
        @Override
        public void onOffsetsChanged(float xOffset, float yOffset, float xStep, float yStep, int xPixels, int yPixels) {
            Log.d("TEST", "onOffsetsChanged:xOffset=" + xOffset + ", yOffset=" + yOffset + ", xStep=" + xStep + ", yStep=" + yStep + ", xPixels=" + xPixels + ", yPixels=" + yPixels);
            drawFrame();
        }
        
        @Override
        public void onVisibilityChanged(boolean visible) {
            Log.d("TEST", "onVisibilityChanged:" + visible);
            this.visible = visible;
            if (visible) {
                drawFrame();
            } else {
                handler.removeCallbacks(drawFire);
            }
        }
        
        @Override
        public void onTouchEvent(MotionEvent event) {
            super.onTouchEvent(event);
            Log.d("TEST", "onTouchEvent");
            if (event.getAction() == MotionEvent.ACTION_DOWN) {
                FireEffect.boost += 10;
            }
        }
        
        private synchronized void drawFrame() {
            Log.d("TEST", "drawFrame[visible:" + visible + "]");
            final SurfaceHolder holder = getSurfaceHolder();
            
            Canvas c = null;
            try {
                c = holder.lockCanvas();
                if (c != null) {
                    fireEffect.draw(c);
                }
            } finally {
                if (c != null) {
                    holder.unlockCanvasAndPost(c);
                }
            }
            
            handler.removeCallbacks(drawFire);
            if (visible) {
                handler.postDelayed(drawFire, 10);
            }
        }
        
        @Override
        public void onSharedPreferenceChanged(SharedPreferences sharedPreferences, String key) {
            if (getResources().getString(R.string.engine_preference_key).equals(key)) {
                fireEffect.useNative = "JNI".equals(prefs.getString(getResources().getString(R.string.engine_preference_key), "JNI"));
            } else {
                fireEffect.init(getWallpaperDesiredMinimumWidth(), getWallpaperDesiredMinimumHeight(), Integer.parseInt(prefs.getString(getResources().getString(R.string.preference_key), "120")));
            }
        }
    }
}
